#!/bin/ruby

require "./Money"

# This file contains the main implementation of the role object pattern
# with Account and Accountrole.
# Please note, that this implementation omits the AccountCore because
# ruby does not directly support interfaces.

class Account

 def initialize(id)
  @id=id.to_i
  @balance=Money::ZeroDollar
 end

 attr_reader :id, :balance

 def plays(roletype)
  false
 end

 def increase(amount)
  raise ArgumentError.new "Amount must always be positive, but was %s"%amount.to_s if Money::ZeroDollar > amount
  @balance+=amount
 end

 def decrease(amount)
  raise ArgumentError.new "Amount must always be positive, but was %s"%amount.to_s if Money::ZeroDollar > amount
  @balance-=amount
 end

 def to_s
  "%d: funds=%s" % [self.id,self.balance.to_s]
 end

end


class AccountRole < Account

 def initialize(base)
  raise ArgumentError.new "base must be an instance of Account but was %s"%base.class.name unless base.kind_of?(Account)
  @base=base
 end

 attr_reader :base

 def increase(amount)
  @base.increase(amount)
 end

 def decrease(amount)
  @base.decrease(amount)
 end

 def balance
  @base.balance
 end

 def id
  @base.id
 end

 def plays(roletype)
  @base.plays(roletype)
 end

 def to_s
  @base.to_s
 end

end
