#!/bin/ruby

require "./Account"

# This file declares new Exceptions, Account roles, and a Bank managing
# these accounts.

class LimitExceeded < StandardError

	def initialize(a,l)
	 @amount=a.to_money
	 @limit=l.to_money
	end

	attr_reader :amount, :limit

	def to_s
	 "The amount %s exceeded the limit of %s" % [@amount.to_s,@limit.to_s]
	end

end

class InsufficientFunds < StandardError

	def to_s
	 "The account had insufficient funds to perform this action"
	end

end

# RoleTypes of Account
class CheckingAccount < AccountRole

	def initialize(base,limit)
	 super(base)
	 @limit=limit.to_money
	end

	attr_reader :limit

	def limit=(value)
	 @limit=value.to_money
	end

	def decrease(amount)
	 raise LimitExceeded.new(amount,limit) if amount > limit
	 self.base.decrease(amount)
	end

	def plays(roletype)
	 roletype==CheckingsAccount or super.plays(roletype)
	end

  def to_s
   "CheckingAccount "+self.base.to_s
  end

end

class SavingsAccount < AccountRole

	def initialize(base,transactionFee)
	 super(base)
	 @transactionFee=transactionFee.to_f
	end

	attr_reader :transactionFee

	def decrease(amount)
		a=amount.to_money * (self.transactionFee+1)
		raise InsufficientFunds.new if a > self.balance
		self.base.decrease(a)
	end

	def plays(roletype)
	 roletype==SavingsAccount or super.plays(roletype)
	end

  def to_s
   "SavingsAccount "+self.base.to_s
  end

end

# The Bank as Context
class Bank

	def initialize
	 @transactions=[]
	 @accounts=[]
	end

	def addSavingsAccount(id)
	 a=find(id)
	 raise ArgumentError.new "Account with the id %d already exists" % id unless (a.nil?)
	 @accounts << SavingsAccount.new(Account.new(id),0.05) 
	end

	def addCheckingAccount(id)
	 a=find(id)
	 raise ArgumentError.new "Account with the id %d already exists" % id.to_i unless (a.nil?)
	 @accounts << CheckingAccount.new(Account.new(id),Money.new(1000,"usd"))
	end

	def find(id)
	 i=id.to_i
	 @accounts.detect{|x| x.id==i }
	end

  def deposite(id,amount)
   a=find(id)
	 raise ArgumentError.new "Account with id %d not found" % from.to_i if a.nil?
   a.increase(amount.to_money)
   nil
  end

	def transfer(from,to,amount)
	 f=find(from)
	 raise ArgumentError.new "Account with id %d not found" % from.to_i if f.nil?
	 t=find(to)
	 raise ArgumentError.new "Account with id %d not found" % to.to_i if t.nil?
	 @transactions << Transaction.new(f,t,amount)
	end

	def executeTransactions
	 (@transactions.select{|t| t.pending? }).sort.each{|t| t.execute }
	end

end
