#!/bin/ruby

require 'bigdecimal'

# This class is a simple representation of money, with infinite 
# precision and arbitrary large numbers.
# It supports addition and subtraction and addition of money as well as
# the multiplication and division with float values.
# These operations ensure that only money in the same concurrency can be
# added or subtracted.
#
# You should resort to a well established money representation for
# example: RubyMoney (https://github.com/RubyMoney/money).

class Money

 def initialize(amount,currency="usd")
   @amount=BigDecimal(amount.to_s)
   @currency=currency.to_sym
 end

 attr_reader :amount, :currency
 attr_writer :amount

 def +(other)
  raise ArgumentError.new "Only Money can be added" unless other.respond_to?(:to_money)
  m=other.to_money
  raise ArgumentError.new "Only Money with the same currency can be added" unless m.currency == self.currency
  Money.new(self.amount + m.amount, self.currency)
 end

 def -(other)
  raise ArgumentError.new "Only Money can be subtracted" unless other.respond_to?(:to_money)
  m=other.to_money
  raise ArgumentError.new "Only Money with the same currency can be subtracted" unless m.currency == self.currency
  Money.new(self.amount - m.amount, self.currency)
 end

 def *(other)
  raise ArgumentError.new "Money can not be multiplied" if other.class==Money
  f=other.to_f
  Money.new(self.amount*f,self.currency)
 end

 def <=>(other)
  m=other.to_money
  raise ArgumentError.new "Only Money with the same currency can be compared" unless m.currency == self.currency
  self.amount <=> m.amount
 end

 include Comparable

 def to_money
  self
 end

 def getFormat
  case self.currency
   when :usd
    "%s $"
   when :eur
    "%s €"
  end
 end

 def to_s
  getFormat() % self.amount.to_s('F')
 end

 ZeroDollar=Money.new("0")

end
