#!/bin/ruby

require 'date'

# This files models two Account roles and a Transaction class used
# within the context of a bank transaction.


class TransactionSource < AccountRole

 def initialize(base)
  super(base) 
 end

 def plays(roletype)
  roletype==TransactionSource or super.plays(roletype)
 end

 def withdraw(amount)
  raise InsufficientFunds.new if (self.balance < amount)  
  self.decrease(amount) 
 end

 def refund(amount)
  self.base.increase(amount)
 end

end


class TransactionSink < AccountRole

 def initialize(base)
  super(base)
 end

 def deposite(amount)
  self.base.increase(amount)
 end

 def plays(roletype)
  roletype==TransactionSink or super.plays(roletype)
 end

end

class Transaction

 def initialize(from,to,amount)
  @creationtime=DateTime.now
  @executiontime=nil
  @amount=amount.to_money
  @from=TransactionSource.new(from)
  @to  =TransactionSink.new(to)
 end

 attr_reader :creationtime, :executiontime
 
 def pending?
  @executiontime.nil?
 end

 def executed?
  !(@executiontime.nil?)
 end

 def execute
  backup=nil
  begin
   @from.withdraw(@amount)
   backup=@amount
   @to.deposite(@amount)
   @executiontime=DateTime.now   
  rescue StandardError => e
   @from.refund(backup) unless backup.nil?   
   $stderr.puts e.to_s
   $stderr.puts e.backtrace
   return false
  end
  true
 end

 include Comparable

 def <=>(other)
  creationtime.<=>(other.creationtime)
 end

end



